--
-- WrapperExtension
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-12-13 - Initial implementation
-- 

WrapperExtension = {};

function WrapperExtension.prerequisitesPresent(specializations)
    return true;
end;

function WrapperExtension:preLoad(savegame)
end;

function WrapperExtension:load(savegame)
	local materialFile = getXMLString(self.xmlFile, "vehicle.WrapperExtension#materialFile");
	
	if materialFile ~= nil and materialFile ~= "" then
		local i3dNode = Utils.loadSharedI3DFile(materialFile, self.baseDirectory, false, false, false);
		
		if i3dNode ~= 0 then
			local materialNode = getChildAt(i3dNode, 0);
			
			if materialNode ~= nil then
				local newMaterial = getMaterial(materialNode, 0);
				
				local i = 0;
				while true do
					local key = string.format("vehicle.baseColorConfigurations.colorNode(%d)", i);
					if not hasXMLProperty(self.xmlFile, key) then break; end;
					
					local node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. "#node"));
					if node ~= nil then
						setMaterial(node, newMaterial, 0);
						print("-- updateing material")
					end;
					
					i = i + 1;
				end;
			end;
		end;
	end;
	
	self.baleColorUpdate = false;
end;

function WrapperExtension:postLoad()
end;

function WrapperExtension:delete()
end;

function WrapperExtension:readStream(streamId, connection)
end;

function WrapperExtension:writeStream(streamId, connection)
end;

function WrapperExtension:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WrapperExtension:keyEvent(unicode, sym, modifier, isDown)
end;

function WrapperExtension:update(dt)
	if self.currentWrapper.currentBale ~= nil then
		if not self.baleColorUpdate then
			local bale = networkGetObject(self.currentWrapper.currentBale);
			
			if bale ~= nil then
				local color = Vehicle.getColorByConfigId(self, "baleColor", self.configurations["baleColor"]);
				local nodes = bale:getMeshNodes();
				
				for _, node in pairs(nodes) do
					if getHasShaderParameter(node, "colorScale") then
						bale.wrapColor = color;
						setShaderParameter(node, "colorScale", color[1], color[2], color[3], color[4], false);
						print("-- updateing bale color")
					end;
				end;
			end;
			
			self.baleColorUpdate = true;
		end;
	else
		self.baleColorUpdate = false;
	end;
end;

function WrapperExtension:updateTick(dt)
end;

function WrapperExtension:draw()
end;



if not Utils.BaleSaveLoadUpdate20161213 then
	Utils.BaleSaveLoadUpdate20161213 = true;

	BaleUpdate = {};
	
	function BaleUpdate:loadUpdate(oldFunc, xmlFile, key, resetVehicles)
		local state = oldFunc(self, xmlFile, key, resetVehicles)
		
		if state then
			local wrapColor = Utils.getVectorNFromString(getXMLString(xmlFile, key .. "#wrapColor"), 3);
			
			if wrapColor ~= nil then
				self.wrapColor = {wrapColor[1], wrapColor[2], wrapColor[3], 1};
				setShaderParameter(self.meshNode, "colorScale", wrapColor[1], wrapColor[2], wrapColor[3], 1, false);
			end;
		end;
		
		return state;
	end;
	
	function BaleUpdate:saveUpdate(oldFunc, nodeIdent)
		local attributes, nodes = oldFunc(self, nodeIdent);
		
		if attributes ~= nil and self.wrapColor ~= nil then
			attributes = attributes .. ' wrapColor="' .. self.wrapColor[1] .. ' ' .. self.wrapColor[2] .. ' ' .. self.wrapColor[3] .. '"';
		end;
		
		return attributes, nodes;
	end;
	
	function BaleUpdate:readStream(oldFunc, streamId, connection)
		oldFunc(self, streamId, connection);
		
		if streamReadBool(streamId) then
			local r = (streamReadUInt8(streamId) / 255);
			local g = (streamReadUInt8(streamId) / 255);
			local b = (streamReadUInt8(streamId) / 255);
			
			self.wrapColor = {r, g, b, 1};
			setShaderParameter(self.meshNode, "colorScale", self.wrapColor[1], self.wrapColor[2], self.wrapColor[3], 1, false);
		end;
	end;
	
	function BaleUpdate:writeStream(oldFunc, streamId, connection)
		oldFunc(self, streamId, connection);
		
		if streamWriteBool(streamId, self.wrapColor ~= nil) then
			streamWriteUInt8(streamId, Utils.clamp(self.wrapColor[1] * 255, 0, 255));
			streamWriteUInt8(streamId, Utils.clamp(self.wrapColor[2] * 255, 0, 255));
			streamWriteUInt8(streamId, Utils.clamp(self.wrapColor[3] * 255, 0, 255));
		end;
	end;
	
	Bale.loadFromAttributesAndNodes = Utils.overwrittenFunction(Bale.loadFromAttributesAndNodes, BaleUpdate.loadUpdate);
	Bale.getSaveAttributesAndNodes = Utils.overwrittenFunction(Bale.getSaveAttributesAndNodes, BaleUpdate.saveUpdate);
	Bale.readStream = Utils.overwrittenFunction(Bale.readStream, BaleUpdate.readStream);
	Bale.writeStream = Utils.overwrittenFunction(Bale.writeStream, BaleUpdate.writeStream);
end;																																																																																																																																																																																																																																																																	print("************ MOD is edit by BR Mod Performance - https://www.facebook.com/BR-Mod-Performance-716359215227674/ *************");																																																																																																																																																																																																																																																																																									


																																									